﻿using System;
using System.Collections.Generic;
using GxPress.Common.Page;

namespace GxPress.Request.Media
{
    /// <summary>
    /// 媒体
    /// </summary>
    public class MediaRequest
    {
        /// <summary>
        /// 媒体类型
        /// Article=1,
        //书籍
        //Book = 2,
        ////课程
        //Curriculum = 3,
        ////音频
        //Audio = 4,
        ////期刊
        //Journal = 5,
        ////视频
        //Video=6
        /// </summary>

        public int MediaType { get; set; }
        /// <summary>
        /// 标题
        /// </summary>

        public string Title { get; set; }

        /// <summary>
        /// 标题图片
        /// </summary>

        public string ImageUrls { get; set; }

        /// <summary>
        /// 简介
        /// </summary>

        public string Summary { get; set; }

        /// <summary>
        /// 作者
        /// </summary>

        public string Author { get; set; }

        /// <summary>
        /// 来源
        /// </summary>

        public string Source { get; set; }

        /// <summary>
        /// Json内容
        /// </summary>
        public string Blocks { get; set; }

        /// <summary>
        /// 审核状态
        /// </summary>

        public bool IsChecked { get; set; }

        /// <summary>
        /// 添加时间
        /// </summary>

        public DateTime AddDate { get; set; }
        /// <summary>
        /// 排序
        /// </summary>

        public int Sort { get; set; }
        /// <summary>
        /// 文章阅读量
        /// </summary>

        public int ReadCount { get; set; }
        /// <summary>
        /// 是否置顶
        /// </summary>

        public bool IsTop { get; set; }
        /// <summary>
        /// 分类ID
        /// </summary>

        public int CategoryId { get; set; }
        /// <summary>
        /// 标签ID
        /// </summary>

        public int LableId { get; set; }
        /// <summary>
        /// 是否推荐
        /// </summary>

        public bool IsRecommend { get; set; }
        /// <summary>
        /// 出版社
        /// </summary>

        public string Press { get; set; }
        /// <summary>
        /// 出版时间
        /// </summary>

        public string PublishTime { get; set; }

        /// <summary>
        /// 媒体类
        /// </summary>
        public List<MediaLibraryRequest> MediaLibraryRequest { get; set; }
        /// <summary>
        /// 媒体标签
        /// </summary>
        public List<MediaLableRequest> MediaLableRequests { get; set; }
    }
    /// <summary>
    /// 媒体库
    /// </summary>
    public class MediaLibraryRequest
    {
        /// <summary>
        /// 媒体ID
        /// </summary>
        /// <value></value>
        public int MediaId { get; set; }
        /// <summary>
        /// 文件路径
        /// </summary>
        public string FileUrl { get; set; }
        /// <summary>
        /// 文件名称
        /// </summary>

        public string Name { get; set; }
        /// <summary>
        /// 排序
        /// </summary>

        public int Sort { get; set; }
    }

    /// <summary>
    /// 媒体标签
    /// </summary>
    public class MediaLableRequest
    {
        /// <summary>
        /// 媒体ID
        /// </summary>
        /// <value></value>
        public int MediaId { get; set; }
        /// <summary>
        /// 标签ID
        /// </summary>
        public string LableId { get; set; }
        /// <summary>
        /// 标签名称
        /// </summary>
        public string LableName { get; set; }
    }
    /// <summary>
    /// 
    /// </summary>
    public class MediaSearchRequest : PageParameter
    {
        /// <summary>
        /// 0全部
        /// </summary>
        /// <value></value>
        public int MediaType { get; set; }
        /// <summary>
        /// 0 全部 1删除 2未删除
        /// </summary>
        /// <value></value>
        public int IsDelete { get; set; }

        /// <summary>
        /// 0 全部 1 包含 2 未包含
        /// </summary>
        /// <value></value>
        public int LibraryType { get; set; }
        /// <summary>
        /// 搜索字段
        /// </summary>
        /// <value></value>
        public string KeyWord { get; set; }
        /// <summary>
        /// 开始时间
        /// </summary>
        /// <value></value>
        public string BeginTime { get; set; }
        /// <summary>
        /// 结束时间
        /// </summary>
        /// <value></value>
        public string EndTime { get; set; }
        /// <summary>
        /// 是否上下架 0全部 1 上架 2下架
        /// </summary>
        /// <value></value>
        public int IsChecked { get; set; }

    }
}
