﻿using System.Linq;
using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.Repository.Interface.Media;
using GxPress.Result.Media;
using Microsoft.Extensions.Options;
using System.Transactions;
using GxPress.Common.Exceptions;
using GxPress.Common.Page;
using GxPress.Request.Media;
using System;

namespace GxPress.Repository.Implement.Media
{
    public class MediaRepository : IMediaRepository
    {
        private readonly Repository<Entity.tede2.Media.Media> _repository;
        private readonly Repository<Entity.tede2.Media.MediaLibrary> mediaLibraryRepository;
        private readonly Repository<Entity.tede2.Media.MediaLable> mediaLableRepository;
        private readonly IMapper _mapper;
        private readonly string _connectionString;
        private readonly string _databaseTypestr;
        public MediaRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper)
        {
            _databaseTypestr = dbOptionsAccessor.CurrentValue.DatabaseType;
            _connectionString = dbOptionsAccessor.CurrentValue.ConnectionString;
            var databaseType =
                StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _repository = new Repository<Entity.tede2.Media.Media>(database);
            mediaLibraryRepository = new Repository<Entity.tede2.Media.MediaLibrary>(database);
            mediaLableRepository = new Repository<Entity.tede2.Media.MediaLable>(database);
            _mapper = mapper;
        }

        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;

        public async Task<MediaResult> GetAsync(int id)
        {
            var result = new MediaResult();
            //获取媒体
            var media = await _repository.GetAsync(id);
            result = _mapper.Map<MediaResult>(media);
            result.ImageUrls = StringUtils.AddDomain(result.ImageUrls);
            result.CategoryId = StringUtils.StringCollectionToStringList(media.CategoryId).ToList();
            result.CategoryName = StringUtils.StringCollectionToStringList(media.CategoryName).ToList();
            result.FullCategoryId = StringUtils.StringCollectionToStringList(media.FullCategoryId).ToList();
            //获取媒体库资源
            var mediaLibrarys = await mediaLibraryRepository.GetAllAsync(Q.Where(nameof(Entity.tede2.Media.MediaLibrary.MediaId), id));
            result.MediaLibraryResults = mediaLibrarys.ToList().Select(n => _mapper.Map<MediaLibraryResult>(n)).ToList();
            foreach (var item in result.MediaLibraryResults)
            {
                item.FileUrl = StringUtils.AddDomain(item.FileUrl);
            }
            //获取媒体标签
            var mediaLables = await mediaLableRepository.GetAllAsync(Q.Where(nameof(Entity.tede2.Media.MediaLable.MediaId), id));
            result.MediaLableResults = mediaLables.Select(n => _mapper.Map<MediaLableResult>(n)).ToList();
            return result;
        }
        /// <summary>
        /// 删除媒体
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteAsync(int id)
        {
            try
            {
                using (var transactionScope = new TransactionScope())
                {
                    // //删除媒体库
                    // await mediaLibraryRepository.DeleteAsync(Q.Where(nameof(Entity.tede2.Media.MediaLibrary.MediaId), id));
                    // //删除媒体标签
                    // await mediaLableRepository.DeleteAsync(Q.Where(nameof(Entity.tede2.Media.MediaLable.MediaId), id));

                    await _repository.UpdateAsync(Q.Where(nameof(Entity.tede2.Media.Media.Id), id).Set(nameof(Entity.tede2.Media.Media.IsDelete), true));
                    transactionScope.Complete();
                }
            }
            catch (System.Exception ex)
            {
                throw new BusinessException(ex.Message);
            }
            return true;
        }
        /// <summary>
        /// 添加媒体
        /// </summary>
        /// <param name="result"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(MediaResult result)
        {
            var model = _mapper.Map<Entity.tede2.Media.Media>(result);
            model.ImageUrls = StringUtils.RemoveDomain(result.ImageUrls);
            try
            {
                using (var transactionScope = new TransactionScope())
                {
                    model.AutoNumber = DateTime.Now.Ticks.ToString();
                    var categoryId = "";
                    foreach (var item in result.CategoryId)
                    {
                        categoryId += item.Split(",")[item.Split(",").Length - 1] + ",";
                    }
                    categoryId = categoryId.Remove(categoryId.Length - 1, 1);
                    model.CategoryId = categoryId;
                    model.FullCategoryId = StringUtils.ObjectCollectionToString(result.CategoryId);
                    var categoryName = "";
                    foreach (var item in result.CategoryName)
                    {
                        categoryName += item.Split(",")[item.Split(",").Length - 1] + ",";
                    }
                    categoryName = categoryName.Remove(categoryName.Length - 1, 1);
                    model.CategoryName = categoryName;
                    model.IsLibrary = result.MediaLibraryResults.Count > 0;
                    var id = await _repository.InsertAsync(model);
                    if (id > 0)
                    {
                        foreach (var item in result.MediaLibraryResults)
                        {
                            var mediaLibrary = _mapper.Map<Entity.tede2.Media.MediaLibrary>(item);
                            mediaLibrary.MediaId = id;
                            mediaLibrary.FileUrl = StringUtils.RemoveDomain(item.FileUrl);
                            await mediaLibraryRepository.InsertAsync(mediaLibrary);
                        }
                    }
                    //添加标签
                    foreach (var item in result.MediaLableResults)
                    {
                        var mediaLable = _mapper.Map<Entity.tede2.Media.MediaLable>(item);
                        mediaLable.MediaId = id;
                        await mediaLableRepository.InsertAsync(mediaLable);
                    }
                    if (result.Id == 0 && result.TeacherId == 0 && !string.IsNullOrEmpty(result.Author))
                    {
                        //新增名师

                    }
                    //提交事务
                    transactionScope.Complete();
                }
            }
            catch (System.Exception ex)
            {
                throw new Common.Exceptions.BusinessException(ex.Message);
            }

            return true;
        }

        public async Task<bool> UpdateAsync(Entity.tede2.Media.Media note)
        {
            return await _repository.UpdateAsync(note);
        }
        public async Task<bool> UpdateAsync(SqlKata.Query query)
        {
            return await _repository.UpdateAsync(query) > 0;
        }
        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PagedList<Entity.tede2.Media.Media>> GetAllAsync(MediaSearchRequest request)
        {
            var result = new PagedList<Entity.tede2.Media.Media>();
            var query = Q.NewQuery();
            if (!string.IsNullOrEmpty(request.KeyWord))
            {
                query.OrWhereLike(nameof(Entity.tede2.Media.Media.Title), $"%{request.KeyWord}%");
                query.OrWhereLike(nameof(Entity.tede2.Media.Media.Creator), $"%{request.KeyWord}%");
            }
            if (request.IsDelete > 0)
                query.Where(nameof(Entity.tede2.Media.Media.IsDelete), request.IsDelete == 1);
            if (request.MediaType > 0)
                query.Where(nameof(Entity.tede2.Media.Media.MediaType), request.MediaType);
            if (request.LibraryType > 0)
                query.Where(nameof(Entity.tede2.Media.Media.IsLibrary), request.LibraryType == 1);
            if (!string.IsNullOrEmpty(request.BeginTime) && !string.IsNullOrEmpty(request.EndTime))
            {
                if (DateTime.TryParse(request.BeginTime, out var beginTime) && DateTime.TryParse(request.EndTime, out var endTime))
                {
                    query.WhereDate(nameof(Entity.tede2.Media.Media.CreatedDate), ">=", beginTime);
                    query.WhereDate(nameof(Entity.tede2.Media.Media.CreatedDate), "<=", endTime);
                }
            }
            if (request.IsChecked > 0)
                query.Where(nameof(Entity.tede2.Media.Media.IsChecked), request.IsChecked == 1);
            result.Total = await _repository.CountAsync(query);
            //排序
            if (string.IsNullOrEmpty(request.Sort))
                query.OrderByDesc(nameof(Entity.tede2.Media.Media.CreatedDate));
            else if (request.Sort.Equals("SellAmount"))
                query.OrderByDesc(nameof(Entity.tede2.Media.Media.SellAmount));
            else if (request.Sort.Equals("ReadCount"))
                query.OrderByDesc(nameof(Entity.tede2.Media.Media.ReadCount));
            var item = await _repository.GetAllAsync(query.ForPage(request.Page, request.PerPage));
            result.Items = item;
            return result;
        }

        public async Task<bool> UpdateAsync(MediaResult result)
        {
            //var model = _mapper.Map<Entity.tede2.Media.Media>(result);
            try
            {
                using (var transactionScope = new TransactionScope())
                {
                    var model = await _repository.GetAsync(result.Id);
                    if (result.CategoryId != null && result.CategoryId.Count > 0)
                    {
                        var categoryId = "";
                        foreach (var item in result.CategoryId)
                            categoryId += item.Split(",")[item.Split(",").Length - 1] + ",";
                        categoryId = categoryId.Remove(categoryId.Length - 1, 1);
                        model.CategoryId = categoryId;
                        model.FullCategoryId = StringUtils.ObjectCollectionToString(result.CategoryId);
                    }
                    if (result.CategoryName != null && result.CategoryName.Count > 0)
                    {
                        var categoryName = "";
                        foreach (var item in result.CategoryName)
                            categoryName += item.Split(",")[item.Split(",").Length - 1] + ",";
                        categoryName = categoryName.Remove(categoryName.Length - 1, 1);
                        model.CategoryName = categoryName;
                    }
                    if (result.AdminId > 0)
                        model.AdminId = result.AdminId;
                    if (!string.IsNullOrEmpty(result.Creator))
                        model.Creator = result.Creator;
                    if (result.MediaType > 0)
                        model.MediaType = result.MediaType;
                    if (!string.IsNullOrEmpty(result.Title))
                        model.Title = result.Title;
                    if (!string.IsNullOrEmpty(result.ImageUrls))
                        model.ImageUrls = result.ImageUrls;
                    if (!string.IsNullOrEmpty(result.Summary))
                        model.Summary = result.Summary;
                    if (!string.IsNullOrEmpty(result.Author))
                        model.Author = result.Author;
                    if (result.TeacherId > 0)
                        model.TeacherId = result.TeacherId;
                    if (!string.IsNullOrEmpty(result.Source))
                        model.Source = result.Source;
                    if (!string.IsNullOrEmpty(result.Blocks))
                        model.Blocks = result.Blocks;
                    if (result.IsChecked > 0)
                        model.IsChecked = result.IsChecked == 1;
                    if (!string.IsNullOrEmpty(result.AddDate))
                        model.AddDate = Convert.ToDateTime(result.AddDate);
                    if (model.Sort > 0)
                        model.Sort = result.Sort;
                    if (model.ReadCount > 0)
                        model.ReadCount = result.ReadCount;
                    if (result.IsTop > 0)
                        model.IsTop = result.IsTop == 1;
                    if (result.LableId > 0)
                        model.LableId = result.LableId;
                    if (result.IsRecommend > 0)
                        model.IsRecommend = result.IsRecommend == 1;
                    if (!string.IsNullOrEmpty(result.Press))
                        model.Press = result.Press;
                    if (!string.IsNullOrEmpty(result.PublishTime))
                        model.PublishTime = result.PublishTime;
                    if (result.IosDiscountPrice > 0)
                        model.IosDiscountPrice = result.IosDiscountPrice;
                    if (result.IosPrice > 0)
                        model.IosPrice = result.IosPrice;
                    if (result.OtherDiscountPrice > 0)
                        model.OtherDiscountPrice = result.OtherDiscountPrice;
                    if (result.OtherPrice > 0)
                        model.OtherPrice = result.OtherPrice;
                    if (result.FreeProportion > 0)
                        model.FreeProportion = result.FreeProportion;
                    if (result.AttachType > 0)
                        model.AttachType = result.AttachType;
                    if (result.AttachId > 0)
                        model.AttachId = result.AttachId;
                    if (result.MediaId > 0)
                        model.MediaId = result.MediaId;
                    if (!string.IsNullOrEmpty(result.MediaTitle))
                        model.MediaTitle = result.MediaTitle;
                    if (!string.IsNullOrEmpty(result.JournalsYear))
                        model.JournalsYear = result.JournalsYear;
                    if (result.JournalsCategory > 0)
                        model.JournalsCategory = result.JournalsCategory;
                    await _repository.UpdateAsync(model);
                    if (result.MediaLibraryResults != null && result.MediaLibraryResults.Count > 0)
                    {
                        model.IsLibrary = result.MediaLibraryResults.Count > 0;
                        //删除媒体库
                        await mediaLibraryRepository.DeleteAsync(Q.Where(nameof(Entity.tede2.Media.MediaLibrary.MediaId), result.Id));
                        foreach (var item in result.MediaLibraryResults)
                        {
                            var mediaLibrary = _mapper.Map<Entity.tede2.Media.MediaLibrary>(item);
                            mediaLibrary.MediaId = result.Id;
                            await mediaLibraryRepository.InsertAsync(mediaLibrary);
                        }
                    }
                    if (result.MediaLableResults != null && result.MediaLableResults.Count > 0)
                    {
                        //删除媒体标签
                        await mediaLableRepository.DeleteAsync(Q.Where(nameof(Entity.tede2.Media.MediaLable.MediaId), result.Id));
                        //添加标签
                        foreach (var item in result.MediaLableResults)
                        {
                            var mediaLable = _mapper.Map<Entity.tede2.Media.MediaLable>(item);
                            mediaLable.MediaId = result.Id;
                            await mediaLableRepository.InsertAsync(mediaLable);
                        }
                    }
                    //提交事务
                    transactionScope.Complete();
                }
            }
            catch (System.Exception ex)
            {
                throw new Common.Exceptions.BusinessException(ex.Message);
            }

            return true;
        }
        public async Task<int> CountAsync(string beginTime, string endTime)
        {
            return await _repository.CountAsync(Q.WhereDate(nameof(Entity.tede2.Media.Media.CreatedDate), ">=", beginTime).WhereDate(nameof(Entity.tede2.Media.Media.CreatedDate), "<=", endTime));
        }
        public async Task<int> CountAsync(SqlKata.Query query)
        {
            return await _repository.CountAsync(query);
        }
    }
}
